# -*- coding: utf-8 -*-

import json
import time

from concurrent.futures import ThreadPoolExecutor

import tornado.web
import tornado.ioloop
import tornado.options
import tornado.httpserver
from tornado.concurrent import run_on_executor

from ydl_ai_recommender.src.utils.log import create_logger
from ydl_ai_recommender.src.core.recommender import UserCFRecommender

logger = create_logger(__name__, 'service.log', is_rotating=True)
recommender = UserCFRecommender(top_n=2, k=50, u2c='order')

class RecommendHandler(tornado.web.RequestHandler):
    executor = ThreadPoolExecutor(1)

    @tornado.gen.coroutine
    def get(self):
        uid = self.get_argument('uid', None)
        if uid is None:
            logger.warn('请求参数不正确，无uid')

        size = self.get_argument('size', 100)
        try:
            size = int(size)
        except Exception as e:
            logger.warn('size=%s 不是数字', size)
            size = 100

        ret = yield self.run(uid, size)
        self.write(ret)


    @tornado.gen.coroutine
    def post(self):
        param = json.loads(self.request.body.decode('utf-8'))
        uid = param.get('uid', None)
        size = param.get('size', 100)
        if uid is None:
            logger.warn('请求参数不正确，无uid')

        ret = yield self.run(uid, size)
        self.write(ret)


    @run_on_executor
    def run(self, uid, size=100):
        logger.info('request@@uid=%s@@size=%s', uid, size)
        try:
            recommend_result = recommender.recommend(uid, size=size, is_merge=True)
            ret = {
                'status': 'success',
                'code': 0,
                'data': recommend_result,
                'total_count': len(recommend_result),
            }
        except Exception as e:
            logger.error('执行推荐函数报错', exc_info=True)
            ret = {
                'status': 'error',
                'code': 1,
                'data': [],
                'total_count': 0,
            }

        ret_str = json.dumps(ret, ensure_ascii=False)
        logger.info('response@@uid=%s@@ret=%s', uid, ret_str)
        return ret

if __name__ == '__main__':

    tornado.options.define('port', default=8868, type=int, help='服务启动的端口号')

    tornado.options.parse_command_line()
    app = tornado.web.Application(handlers=[(r'/ai_counselor_recommend', RecommendHandler), ]
                                  , autoreload=False, debug=False)
    http_server = tornado.httpserver.HTTPServer(app)
    http_server.listen(tornado.options.options.port)
    tornado.ioloop.IOLoop.instance().start()
